@@ -9,7 +9,7 @@ import sys
 # add parent folder to python path so that we can import noxfile_utils.py
 # note that you need to "pip install -r noxfile-requiterements.txt" for this file to work.
 sys.path.append(str(Path(__file__).parent / "ci_tools"))
-from nox_utils import PY27, PY37, PY36, PY35, PY38, power_session, rm_folder, rm_file, PowerSession  # noqa
+from nox_utils import PY27, PY37, PY36, PY35, PY38, PY39, PY310, power_session, rm_folder, rm_file, PowerSession  # noqa
 
 
 pkg_name = "makefun"
@@ -17,6 +17,9 @@ gh_org = "smarie"
 gh_repo = "python-makefun"
 
 ENVS = {
+    # python 3.10 is not available on conda yet
+    # PY310: {"coverage": False, "pkg_specs": {"pip": ">19"}},
+    PY39: {"coverage": False, "pkg_specs": {"pip": ">19"}},
     PY38: {"coverage": False, "pkg_specs": {"pip": ">19"}},
     PY27: {"coverage": False, "pkg_specs": {"pip": ">10"}},
     PY35: {"coverage": False, "pkg_specs": {"pip": ">10"}},
@@ -27,7 +30,7 @@ ENVS = {
 
 
 # set the default activated sessions, minimal for CI
-nox.options.sessions = ["tests"]  # , "docs", "gh_pages"
+nox.options.sessions = ["tests", "flake8"]  # , "docs", "gh_pages"
 nox.options.reuse_existing_virtualenvs = True  # this can be done using -r
 # if platform.system() == "Windows":  >> always use this for better control
 nox.options.default_venv_backend = "conda"
@@ -40,6 +43,7 @@ nox_logger = logging.getLogger("nox")
 
 class Folders:
     root = Path(__file__).parent
+    ci_tools = root / "ci_tools"
     runlogs = root / Path(nox.options.envdir or ".nox") / "_runlogs"
     runlogs.mkdir(parents=True, exist_ok=True)
     dist = root / "dist"
@@ -47,8 +51,16 @@ class Folders:
     site_reports = site / "reports"
     reports_root = root / "docs" / "reports"
     test_reports = reports_root / "junit"
+    test_xml = test_reports / "junit.xml"
+    test_html = test_reports / "report.html"
+    test_badge = test_reports / "junit-badge.svg"
     coverage_reports = reports_root / "coverage"
     coverage_xml = coverage_reports / "coverage.xml"
+    coverage_intermediate_file = root / ".coverage"
+    coverage_badge = coverage_reports / "coverage-badge.svg"
+    flake8_reports = reports_root / "flake8"
+    flake8_intermediate_file = root / "flake8stats.txt"
+    flake8_badge = flake8_reports / "flake8-badge.svg"
 
 
 @power_session(envs=ENVS, logsdir=Folders.runlogs)
@@ -59,7 +71,7 @@ def tests(session: PowerSession, coverage, pkg_specs):
     rm_folder(Folders.site)
     rm_folder(Folders.reports_root)
     # delete the .coverage files if any (they are not supposed to be any, but just in case)
-    rm_file(Folders.root / ".coverage")
+    rm_file(Folders.coverage_intermediate_file)
     rm_file(Folders.root / "coverage.xml")
 
     # CI-only dependencies
@@ -93,32 +105,59 @@ def tests(session: PowerSession, coverage, pkg_specs):
 
     # install self so that it is recognized by pytest
     session.run2("pip install -e . --no-deps")
+    # session.install("-e", ".", "--no-deps")
 
     # check that it can be imported even from a different folder
-    session.run2(['python', '-c', '"import os; os.chdir(\'./docs/\'); import %s"' % pkg_name])
+    # Important: do not surround the command into double quotes as in the shell !
+    session.run('python', '-c', 'import os; os.chdir(\'./docs/\'); import %s' % pkg_name)
 
     # finally run all tests
     if not coverage:
         # simple: pytest only
-        session.run2("python -m pytest -v %s/tests/" % pkg_name)
+        session.run2("python -m pytest --cache-clear -v tests/")
     else:
         # coverage + junit html reports + badge generation
-        session.install_reqs(phase="coverage", phase_reqs=["coverage", "pytest-html", "requests", "xunitparser"],
+        session.install_reqs(phase="coverage",
+                             phase_reqs=["coverage", "pytest-html", "genbadge[tests,coverage]"],
                              versions_dct=pkg_specs)
 
         # --coverage + junit html reports
-        session.run2("coverage run --source {pkg_name} "
-                     "-m pytest --junitxml={dst}/junit.xml --html={dst}/report.html -v {pkg_name}/tests/"
-                     "".format(pkg_name=pkg_name, dst=Folders.test_reports))
-        # session.run2("coverage report")  # this shows in terminal + fails under XX%, same as --cov-report term --cov-fail-under=70  # noqa
+        session.run2("coverage run --source src/{pkg_name} "
+                     "-m pytest --cache-clear --junitxml={test_xml} --html={test_html} -v tests/"
+                     "".format(pkg_name=pkg_name, test_xml=Folders.test_xml, test_html=Folders.test_html))
+        session.run2("coverage report")
         session.run2("coverage xml -o {covxml}".format(covxml=Folders.coverage_xml))
         session.run2("coverage html -d {dst}".format(dst=Folders.coverage_reports))
         # delete this intermediate file, it is not needed anymore
-        rm_file(Folders.root / ".coverage")
+        rm_file(Folders.coverage_intermediate_file)
 
         # --generates the badge for the test results and fail build if less than x% tests pass
         nox_logger.info("Generating badge for tests coverage")
-        session.run2("python ci_tools/generate-junit-badge.py 100 %s" % Folders.test_reports)
+        # Use our own package to generate the badge
+        session.run2("genbadge tests -i %s -o %s -t 100" % (Folders.test_xml, Folders.test_badge))
+        session.run2("genbadge coverage -i %s -o %s" % (Folders.coverage_xml, Folders.coverage_badge))
+
+
+@power_session(python=PY38, logsdir=Folders.runlogs)
+def flake8(session: PowerSession):
+    """Launch flake8 qualimetry."""
+
+    session.install("-r", str(Folders.ci_tools / "flake8-requirements.txt"))
+    session.install("genbadge[flake8]")
+    session.run2("pip install -e .[flake8]")
+
+    rm_folder(Folders.flake8_reports)
+    Folders.flake8_reports.mkdir(parents=True, exist_ok=True)
+    rm_file(Folders.flake8_intermediate_file)
+
+    session.cd("src")
+
+    # Options are set in `setup.cfg` file
+    session.run("flake8", pkg_name, "--exit-zero", "--format=html", "--htmldir", str(Folders.flake8_reports),
+                "--statistics", "--tee", "--output-file", str(Folders.flake8_intermediate_file))
+    # generate our badge
+    session.run2("genbadge flake8 -i %s -o %s" % (Folders.flake8_intermediate_file, Folders.flake8_badge))
+    rm_file(Folders.flake8_intermediate_file)
 
 
 @power_session(python=[PY37])
@@ -129,9 +168,9 @@ def docs(session: PowerSession):
 
     if session.posargs:
         # use posargs instead of "serve"
-        session.run2("mkdocs -f ./docs/mkdocs.yml %s" % " ".join(session.posargs))
+        session.run2("mkdocs %s" % " ".join(session.posargs))
     else:
-        session.run2("mkdocs serve -f ./docs/mkdocs.yml")
+        session.run2("mkdocs serve")
 
 
 @power_session(python=[PY37])
@@ -141,14 +180,14 @@ def publish(session: PowerSession):
     session.install_reqs(phase="mkdocs", phase_reqs=["mkdocs-material", "mkdocs", "pymdown-extensions", "pygments"])
 
     # possibly rebuild the docs in a static way (mkdocs serve does not build locally)
-    session.run2("mkdocs build -f ./docs/mkdocs.yml")
+    session.run2("mkdocs build")
 
     # check that the doc has been generated with coverage
     if not Folders.site_reports.exists():
         raise ValueError("Test reports have not been built yet. Please run 'nox -s tests-3.7' first")
 
     # publish the docs
-    session.run2("mkdocs gh-deploy -f ./docs/mkdocs.yml")
+    session.run2("mkdocs gh-deploy")
 
     # publish the coverage - now in github actions only
     # session.install_reqs(phase="codecov", phase_reqs=["codecov", "keyring"])
@@ -210,7 +249,7 @@ def release(session: PowerSession):
     session.install_reqs(phase="release", phase_reqs=["click", "PyGithub"])
     session.run2("python ci_tools/github_release.py -s {gh_token} "
                  "--repo-slug {gh_org}/{gh_repo} -cf ./docs/changelog.md "
-                 "-d https://{gh_org}.github.io/{gh_repo}/changelog/ {tag}"
+                 "-d https://{gh_org}.github.io/{gh_repo}/changelog {tag}"
                  "".format(gh_token=gh_token, gh_org=gh_org, gh_repo=gh_repo, tag=current_tag))
 
 
